/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Copyright 2002-2004 Todd Kulesza
 *
 * Authors:
 * 		Todd Kulesza <todd@dropline.net>
 */

#include <stdio.h>
#include <unistd.h>
#include <gnome.h>
#include <glade/glade-xml.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <string.h>
#include <pwd.h>
#include <curl/curl.h>

#include "insert_poll_dialog.h"
#include "query_music_players.h"
#include "tray.h"
#include "drivel.h"
#include "about.h"
#include "login.h"
#include "network.h"
#include "dialogs.h"
#include "journal.h"

#include <config.h>

extern GMutex *network_mutex;

static gchar *
get_home_dir (void)
{
	gchar *dir;
	struct passwd *pass;
	
	dir = NULL;
	
	if ( (pass = getpwuid (geteuid ())) )
		dir = g_strdup (pass->pw_dir);
	
	return dir;
}

void
remove_autosave (DrivelClient *dc)
{
	gchar *dir, *autosave;
	
	dir = get_home_dir ();
	autosave = g_strdup_printf ("%s/.drivel_autosave", dir);
	
	unlink (autosave);
	
	dc->modified_autosave = FALSE;
	
	g_free (autosave);
	
	return;
}

static gint
get_line_len (const gchar *string, gint len, gint offset)
{
	gint i;
	
	for (i = offset; i < len; i++)
	{
		if (string [i] == '\n')
			return i - offset;
	}

	return i - offset;
}

static void
open_file (const gchar *filename, DrivelClient *dc)
{
	gchar input_data [4096], value [4096], *autosave, *dir;
	gint offset, len, item;
	gboolean state;
	GnomeVFSHandle *handle;
	GnomeVFSFileSize bytes_read;
	GnomeVFSResult result;
	GtkTextBuffer *buffer;

	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dc->journal_text));
	
	/* if 'filename' is not null, this was called from the "Open Draft" dialog.
	 * otherwise, it was called by the autosave function. */
	if (filename)
		result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
	else
	{
		if ( (dir = get_home_dir ()) )
		{
			autosave = g_strdup_printf ("%s/.drivel_autosave", dir);
			result = gnome_vfs_open (&handle, autosave, GNOME_VFS_OPEN_READ);
			if (result != GNOME_VFS_OK)
				return;
			else
				dc->modified = TRUE;
		}
		else
			return;
		g_free (autosave);
	}
	
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_seek (handle, GNOME_VFS_SEEK_START, 0);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_read (handle, input_data, 4096, &bytes_read);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_close (handle);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	offset = 0;
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	gtk_entry_set_text (GTK_ENTRY (dc->journal_subject), value);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_mood)->child), value);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_music)->child), value);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	item = (gint) g_ascii_strtod (value, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_security), item);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	item = (gint) g_ascii_strtod (value, NULL);
	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_picture), item);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	state = (gboolean) g_ascii_strtod (value, NULL);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_comment), state);
	
	len = get_line_len (input_data, bytes_read, offset);
	memcpy (value, input_data + offset, len);
	value [len] = '\0';
	offset += len + 1;
	state = (gboolean) g_ascii_strtod (value, NULL);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat), state);
	
	memcpy (value, input_data + offset, bytes_read - offset);
	value [bytes_read - offset] = '\0';
	gtk_text_buffer_set_text (buffer, value, strlen (value));
	
	dc->modified = FALSE;
	remove_autosave (dc);
	
	return;
}

static void
open_draft_cb (GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *parent;
	GtkFileFilter *filter;
	DrivelClient *dc = (DrivelClient *) data;
	
	parent = drivel_get_current_window (dc->window_list);
	
	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("All Text Files"));
	gtk_file_filter_add_mime_type (filter, "text/plain");
	
	dialog = gtk_file_chooser_dialog_new (_("Drivel - Open Draft"), 
										  GTK_WINDOW (parent), 
										  GTK_FILE_CHOOSER_ACTION_OPEN,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
										  NULL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *filename;
		
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		open_file (filename, dc);
		g_free (filename);
	}
	
	gtk_widget_destroy (dialog);
	
	return;
}

static void
save_file (const gchar *filename, DrivelClient *dc)
{
	gchar *output_data, *text, *subject, *mood, *music;
	gchar *autosave, *dir;
	gint security, picture, comment, autoformat;
	GnomeVFSHandle *handle;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	GnomeVFSFileSize written;
	GnomeVFSResult result;
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dc->journal_text));
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	if (!text)
		text = g_strdup ("");
	
	subject = gtk_editable_get_chars (GTK_EDITABLE (dc->journal_subject), 0, -1);
	if (!subject)
		subject = g_strdup ("");
	mood = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (dc->journal_mood)->child), 0, -1);
	if (!mood)
		mood = g_strdup ("");
	music = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (dc->journal_music)->child), 0, -1);
	if (!music)
		music = g_strdup ("");
	security = gtk_combo_box_get_active (GTK_COMBO_BOX (dc->journal_security));
	picture = gtk_combo_box_get_active (GTK_COMBO_BOX (dc->journal_picture));
	comment = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->journal_comment));
	autoformat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat));
	
	output_data = g_strdup_printf (
			"%s\n"
			"%s\n"
			"%s\n"
			"%d\n"
			"%d\n"
			"%d\n"
			"%d\n"
			"%s\n",
			subject, mood, music, security, picture,
			comment, autoformat, text);
	
	/* if 'filename' is not null, this was called from the "Save Draft" dialog.
	 * otherwise, it was called by the autosave function. */
	if (filename)
	{
		result = gnome_vfs_create (&handle, filename, GNOME_VFS_OPEN_WRITE, FALSE,
			GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE);
	}
	else
	{
		if ( (dir = get_home_dir ()) )
		{
			autosave = g_strdup_printf ("%s/.drivel_autosave", dir);
			result = gnome_vfs_create (&handle, autosave, GNOME_VFS_OPEN_WRITE, FALSE,
					GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE);
		}
		else
			return;
		g_free (autosave);
	}
	
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_seek (handle, GNOME_VFS_SEEK_START, 0);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_write (handle, output_data, strlen (output_data), &written);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	result = gnome_vfs_close (handle);
	if (result != GNOME_VFS_OK)
	{
		display_gnomevfs_error_dialog (dc, result);
		
		return;
	}
	
	if (filename)
	{
		dc->modified = FALSE;
		remove_autosave (dc);
	}
	
	g_free (output_data);
	g_free (text);
	g_free (subject);
	g_free (mood);
	g_free (music);
	
	return;
}

static void
save_draft (DrivelClient *dc)
{
	GtkWidget *dialog;
	GtkWidget *parent;
	GtkFileFilter *filter;
	
	parent = drivel_get_current_window (dc->window_list);
	
	filter = gtk_file_filter_new ();
	gtk_file_filter_set_name (filter, _("All Text Files"));
	gtk_file_filter_add_mime_type (filter, "text/plain");

	dialog = gtk_file_chooser_dialog_new (_("Drivel - Save Draft"),
										  GTK_WINDOW (parent),
										  GTK_FILE_CHOOSER_ACTION_SAVE,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
										  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
										  NULL);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
	
	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *filename;
		
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (dc->draft_filename)
			g_free (dc->draft_filename);
		dc->draft_filename = filename;
		save_file (filename, dc);
	}
	
	gtk_widget_destroy (dialog);
	
	return;
}

void
save_draft_cb (GtkWidget *widget, gpointer data)
{
	
	DrivelClient *dc = (DrivelClient *) data;
	
	if (dc->draft_filename)
		save_file (dc->draft_filename, dc);
	else
		save_draft (dc);

	return;
}

static gboolean
autosave_cb (gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	if (!dc->modified)
		return TRUE;
	
	save_file (NULL, dc);
	
	return TRUE;
}

static void
post_entry (DrivelClient *dc, DrivelRequestType mode)
{
	gchar *post_data, *text, *mood;
	gchar *music, *subject, *pic, *security, *key;
	gchar *mode_specific, *mesg;
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	gint i, allowmask, mood_id;
	gboolean autoformat, no_comments;
	gchar *vars [11], *values [11];
	gchar *active_journal;
				
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dc->journal_text));
	
	gtk_text_buffer_get_bounds (buffer, &start,& end);
	
	allowmask = 0;
	text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	mood = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (dc->journal_mood)->child), 0, -1);
	music = gtk_editable_get_chars (GTK_EDITABLE (GTK_BIN (dc->journal_music)->child), 0, -1);
	subject = gtk_editable_get_chars (GTK_EDITABLE (dc->journal_subject), 0, -1);
	i = gtk_combo_box_get_active (GTK_COMBO_BOX (dc->journal_security));
	switch (i)
	{
		case 2:
		{
			security = g_strdup ("private");
			break;
		}
		case 1:
		{
			security = g_strdup ("usemask");
			allowmask = 1;
			break;
		}
		default:
			security = g_strdup ("public");
	}
	i = gtk_combo_box_get_active (GTK_COMBO_BOX (dc->journal_picture));
	key = g_strdup_printf ("pickw_%d", i);
	pic = g_hash_table_lookup (dc->picture_keywords, key);
	autoformat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat));
	no_comments = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->journal_comment));
	
	if (mood && (mesg = g_hash_table_lookup (dc->mood_icons, mood)))
		mood_id = (gint) g_ascii_strtod (mesg, NULL);
	else
		mood_id = 0;
	
	switch (mode)
	{
		case REQUEST_POSTEVENT:
		{
			fill_time (dc);
			
			mode_specific = g_strdup_printf (
					"mode=postevent");
			break;
		}
		case REQUEST_EDITEVENT:
		{
			mode_specific = g_strdup_printf (
					"mode=editevent&"
					"itemid=%s",
					dc->itemid);
			break;
		}
		default:
		{
			mode_specific = NULL;
			g_assert_not_reached ();
			break;
		}
	}
	
	vars [0] = NULL;
	vars [1] = NULL;
	vars [2] = NULL;
	vars [3] = lj_format_string ("prop_current_music");
	vars [4] = lj_format_string ("prop_current_mood");
	vars [5] = lj_format_string ("prop_current_moodid");
	vars [6] = NULL;
	vars [7] = lj_format_string ("prop_picture_keyword");
	vars [8] = lj_format_string ("prop_opt_nocomments");
	vars [9] = lj_format_string ("prop_opt_preformatted");
	vars [10] = NULL;
	values [0] = lj_format_string (dc->username);
	values [1] = NULL;
	values [2] = lj_format_string (text);
	values [3] = lj_format_string (music);
	values [4] = lj_format_string (mood);
	values [5] = NULL;
	values [6] = lj_format_string (subject);
	values [7] = lj_format_string (pic);
	values [8] = NULL;
	values [9] = NULL;
	values [10] = lj_format_string (dc->active_journal);
	
	if (dc->active_journal)
		active_journal = g_strdup_printf ("&usejournal=%s", values [10]);
	else
		active_journal = g_strdup ("");
	
	post_data = g_strdup_printf (
			"%s&ver=1&user=%s&hpassword=%s&"
			"event=%s&lineendings=unix&"
			"%s=%s&%s=%s&"
			"%s=%d&subject=%s&"
			"security=%s&allowmask=%d&"
			"%s=%s&"
			"year=%04d&mon=%02d&day=%02d&"
			"hour=%02d&min=%02d&"
			"%s=%d&"
			"%s=%d"
			"%s",
			mode_specific, values [0], dc->password,
			values [2],
			vars [3], values [3], vars [4], values [4],
			vars [5], mood_id, values [6],
			security, allowmask, vars [7], values [7],
			dc->time.year, dc->time.month,
			dc->time.day, dc->time.hour, dc->time.minute,
			vars [8], no_comments,
			vars [9], autoformat,
			active_journal);

	lj_send_request (dc, post_data, REQUEST_POSTEVENT);

	g_free (mode_specific);	
	g_free (text);
	g_free (mood);
	g_free (music);
	g_free (subject);
	g_free (security);
	g_free (active_journal);
	
	for (i = 0; i < 11; i++)
	{
		if (vars [i])
			curl_free (vars [i]);
		if (values [i])
			curl_free (values [i]);
	}
			
	return;
}

static void
journal_display_defaults (DrivelClient *dc)
{
	gchar *string;
	GtkTextIter start, end;
	
	/* if the user saved a draft, clear it's filename so that she gets prompted
	   again. */
	if (dc->draft_filename)
	{
		g_free (dc->draft_filename);
		dc->draft_filename = NULL;
	}
	
	gtk_text_buffer_get_bounds (dc->buffer, &start, &end);
	
	gtk_text_buffer_delete (dc->buffer, &start, &end);
	
	gtk_entry_set_text (GTK_ENTRY (dc->journal_subject), "");

	string = get_default_text (dc->client, dc->gconf->default_mood, "");
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_mood)->child), string);
	g_free (string);

	string = get_default_text (dc->client, dc->gconf->default_music, "");
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_music)->child), string);
	g_free (string);
	
	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_picture),
			gconf_client_get_int (dc->client, dc->gconf->default_picture, NULL));
	
	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_security),
			gconf_client_get_int (dc->client, dc->gconf->default_security, NULL));
	
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_comment),
			gconf_client_get_bool (dc->client, dc->gconf->default_comment, NULL));
	
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat),
			gconf_client_get_bool (dc->client, dc->gconf->default_autoformat, NULL));
	
	gtk_widget_set_sensitive (dc->menu_cut, FALSE);
	gtk_widget_set_sensitive (dc->menu_copy, FALSE);
	
	gtk_widget_grab_focus (dc->journal_text);
	
	return;
}

void
journal_edit_entry (DrivelClient *dc, const gchar *itemid, const gchar *event, const gchar *security,
		const gchar *allowmask, const gchar *subject, const gchar *mood, const gchar *music, const gchar *picture,
		const gchar *eventtime, const gchar *comments, const gchar *autoformat)
{
	gboolean bool_comments, bool_autoformat;
	gchar time [5], *pickw, *text;
	gint item, i;
	
	time [4] = '\0';
	
	memcpy (time, eventtime, 4);
	dc->time.year = (gint) g_ascii_strtod (time, NULL);
	time [2] = '\0';
	memcpy (time, eventtime + 5, 2);
	dc->time.month = (gint) g_ascii_strtod (time, NULL);
	memcpy (time, eventtime + 8, 2);
	dc->time.day = (gint) g_ascii_strtod (time, NULL);
	memcpy (time, eventtime + 11, 2);
	dc->time.hour = (gint) g_ascii_strtod (time, NULL);
	memcpy (time, eventtime + 14, 2);
	dc->time.minute = (gint) g_ascii_strtod (time, NULL);
	
	if (!security)
		item = 0;
	else if (!strcmp ("private", security))
		item = 2;
	else
		item = 1;

	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_security), item);
	
	if (comments)
		bool_comments = (gboolean) g_ascii_strtod (comments, NULL);
	else
		bool_comments = FALSE;
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_comment),
			bool_comments);
	
	if (autoformat)
		bool_autoformat = (gboolean) g_ascii_strtod (autoformat, NULL);
	else
		bool_autoformat = FALSE;
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat),
			bool_autoformat);

	if (picture)
	{
		for (i = 0; i < dc->pictures + 1; i++)
		{
			pickw = g_strdup_printf ("pickw_%d", i);
			text = g_hash_table_lookup (dc->picture_keywords, pickw);

			if (!strcmp (picture, text))
				gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_picture), i);

			g_free (pickw);
		}
	}
	else
		gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_picture), 0);
		
	if (!event)
		event = g_strdup ("");
	if (!subject)
		subject = g_strdup ("");
	if (!mood)
		mood = g_strdup ("");
	if (!music)
		music = g_strdup ("");
		
	gtk_text_buffer_set_text (dc->buffer, event, -1);
	
	gtk_entry_set_text (GTK_ENTRY (dc->journal_subject), subject);
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_mood)->child), mood);
	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_music)->child), music);
	
	dc->itemid = g_strdup (itemid);
	
	gtk_widget_hide (dc->journal_post);
	gtk_widget_show (dc->edit_delete);
	gtk_widget_show (dc->edit_save);
	gtk_widget_show (dc->edit_cancel);
	
/*	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_options), TRUE); */
	
	gtk_widget_grab_default (dc->edit_save);
	gtk_widget_grab_focus (dc->journal_text);
	
	dc->modified = FALSE;
	
	return;
}

void
journal_edit_entry_finished (DrivelClient *dc)
{
	g_free (dc->itemid);
	dc->itemid = NULL;
	
	gtk_widget_hide (dc->edit_delete);
	gtk_widget_hide (dc->edit_save);
	gtk_widget_hide (dc->edit_cancel);
	gtk_widget_show (dc->journal_post);
		
/*	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_options), FALSE); */
	
	journal_display_defaults (dc);
	
	return;
}
/*
void
server_list_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gint item;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	item = gconf_value_get_int (value);
	
	gtk_option_menu_set_history (GTK_OPTION_MENU (dc->pref_server_list), item);
	
	return;
}
*/

static void
server_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *string;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	string = gconf_value_get_string (value);
	
	g_free (dc->server);
	dc->server = g_strdup (string);

	lj_http_reslv_name (dc);
	
	return;
}

static void
proxy_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gboolean state;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	state = gconf_value_get_bool (value);
	
	g_mutex_lock (network_mutex);
	dc->proxy = state;
	g_mutex_unlock (network_mutex);
	
	if (state)
		lj_http_reslv_name (dc);
	
	return;
}

static void
proxy_user_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *user;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	user = gconf_value_get_string (value);
	
	g_mutex_lock (network_mutex);
	g_free (dc->proxy_user);
	dc->proxy_user = g_strdup (user);
	g_mutex_unlock (network_mutex);
	
	return;
}

static void
proxy_pass_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *pass;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	pass = gconf_value_get_string (value);
	
	g_mutex_lock (network_mutex);
	g_free (dc->proxy_pass);
	dc->proxy_pass = g_strdup (pass);
	g_mutex_unlock (network_mutex);
	
	return;
}

static void
proxy_url_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *url;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	url = gconf_value_get_string (value);
	
	g_mutex_lock (network_mutex);
	g_free (dc->proxy_url);
	dc->proxy_url = g_strdup (url);
	g_mutex_unlock (network_mutex);
	
	lj_http_reslv_name (dc);
	
	return;
}

static void
proxy_port_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gint port;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	port = gconf_value_get_int (value);
	
	g_mutex_lock (network_mutex);
	dc->proxy_port = port;
	g_mutex_unlock (network_mutex);
	
	return;
}

static void
tray_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gboolean state;
	DrivelClient *dc = (DrivelClient *) data;

	value = gconf_entry_get_value (entry);
	state = gconf_value_get_bool (value);
	
	dc->tray = state;
	
	if (state && dc->journal_window)
		tray_create (dc);
	else
		tray_destroy (dc);
	
	return;
}

static void
mood_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *string;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	string = gconf_value_get_string (value);

	if (dc->journal_mood)
		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_mood)->child),
				string);
	
	return;
}

static void
music_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	const gchar *string;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	string = gconf_value_get_string (value);
	
	if (dc->journal_music)
		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_music)->child), string);
	
	return;
}

static void
picture_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gint num;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	num = gconf_value_get_int (value);
	
	if (dc->journal_picture)
		gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_picture), num);
	
	return;
}

static void
security_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gint num;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	num = gconf_value_get_int (value);
	
	if (dc->journal_security)
		gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_security), num);
	
	return;
}

static void
comment_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gboolean state;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	state = gconf_value_get_bool (value);

	if (dc->journal_comment)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_comment),
				state);

	return;
}

static void
autoformat_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
{
	GConfValue *value;
	gboolean state;
	DrivelClient *dc = (DrivelClient *) data;
	
	value = gconf_entry_get_value (entry);
	state = gconf_value_get_bool (value);

	if (dc->journal_autoformat)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dc->journal_autoformat),
				state);

	return;
}

gboolean
delete_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	gboolean retval;

	retval = FALSE;

	journal_window_state_save (dc);
	
	display_save_dialog (dc);

	curl_global_cleanup ();

	gtk_main_quit ();
	
	return retval;
}

void
exit_cb (GtkWidget *widget, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;

	if (dc->journal_window)
		display_save_dialog (dc);
	
	gtk_main_quit ();
	
	return;
}

static void
logout_cb (GtkWidget *widget, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;

	display_save_dialog (dc);
	
	journal_window_state_save (dc);
	
	gtk_widget_destroy (dc->journal_window);
	gtk_widget_show_all (dc->login_window);
	drivel_pop_current_window (dc);
	dc->journal_window = NULL;
	
	if (dc->picture_keywords)
	{
		hash_table_clear (dc->picture_keywords);
		dc->picture_keywords = NULL;
	}
	
	if (dc->journal_list)
	{
		g_slist_foreach (dc->journal_list, list_free_item, NULL);
		g_slist_free (dc->journal_list);
		dc->journal_list = NULL;
	}
	
	if (dc->menu_list)
	{
		g_slist_foreach (dc->menu_list, menu_list_free_item, NULL);
		g_slist_free (dc->menu_list);
		dc->menu_list = NULL;
	}
	
	g_source_remove (dc->tag_autosave);
	
	drivel_fill_journal_null (dc);
	
	/* clear the password if the user doesn't want it saved */
	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dc->save_password)))
	{
		gtk_entry_set_text (GTK_ENTRY (dc->login_password), "");
		g_free (dc->password);
		dc->password = NULL;
	}
	
	tray_destroy (dc);
	
	gtk_editable_select_region (GTK_EDITABLE (GTK_BIN (dc->login_name)->child), 0, -1);
	gtk_widget_grab_focus (GTK_BIN (dc->login_name)->child);

	return;
}

static void
text_changed_cb (GtkWidget *widget, gpointer data)
{
	gint text;
	DrivelClient *dc = (DrivelClient *) data;
	
	
	text = gtk_text_buffer_get_char_count (GTK_TEXT_BUFFER (widget));
	
	if (text)
	{
		dc->modified = TRUE;
		gtk_widget_set_sensitive (dc->journal_post, TRUE);
	}
	else
	{
		dc->modified = FALSE;
		gtk_widget_set_sensitive (dc->journal_post, FALSE);
	}
	
	return;
}

static void
insert_html (DrivelClient *dc, const gchar *open, const gchar *close)
{
	GtkTextIter start, end;
	GtkTextMark *mark_insert, *mark_selection;
	gboolean reverse = FALSE;

	mark_selection = gtk_text_buffer_get_selection_bound (dc->buffer);
	mark_insert = gtk_text_buffer_get_insert (dc->buffer);
	
	gtk_text_buffer_get_iter_at_mark (dc->buffer, &start, mark_selection);
	gtk_text_buffer_get_iter_at_mark (dc->buffer, &end, mark_insert);
	if (gtk_text_iter_compare (&start, &end) > 0)
	{
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &start, mark_insert);
		reverse = TRUE;
	}
	gtk_text_buffer_insert (dc->buffer, &start, open, -1);

	if (!close)
		return;
		
	if (reverse)
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &end, mark_selection);
	else
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &end, mark_insert);
	gtk_text_buffer_insert (dc->buffer, &end, close, -1);
	
	if (reverse)
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &end, mark_selection);
	else
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &end, mark_insert);
	gtk_text_iter_backward_chars (&end, strlen (close));
	gtk_text_buffer_place_cursor (dc->buffer, &end);
	
	return;	
}

static void
skip_tag (DrivelClient *dc, gchar *tag)
{
	gchar *text, *search_text, *search_tag;
	GtkTextMark *mark_insert;
	GtkTextIter start, end;
	gint characters = 0, matched = 0, to_match;
	gunichar ch1, ch2;
	
	mark_insert = gtk_text_buffer_get_insert (dc->buffer);
	gtk_text_buffer_get_iter_at_mark (dc->buffer, &start, mark_insert);
	gtk_text_buffer_get_end_iter (dc->buffer, &end);
		
	text = gtk_text_buffer_get_text (dc->buffer, &start, &end, FALSE);
	search_text = text;
	search_tag = tag;
	ch2 = g_utf8_get_char (search_tag);
	to_match = g_utf8_strlen (tag, -1);
	
	while (*search_text)
	{
		characters++;
		ch1 = g_utf8_get_char (search_text);
		if (ch1 == ch2)
		{
			if (++matched == to_match)
				break;
			
			search_tag = g_utf8_next_char (search_tag);
			ch2 = g_utf8_get_char (search_tag);
		}
		else if (matched)
		{
			matched = 0;
			search_tag = tag;
			ch2 = g_utf8_get_char (search_tag);
		}
		
		search_text = g_utf8_next_char (search_text);
		ch1 = g_utf8_get_char (search_text);
	}
	
	gtk_text_iter_forward_chars (&start, characters);
	gtk_text_buffer_place_cursor (dc->buffer, &start);
	
	g_free (text);
	
	return;
}

static gboolean
inside_tag (DrivelClient *dc, const gchar *open, const gchar *close)
{
	GtkTextIter start, end;
	GtkTextMark *mark_insert;
	gchar *text, *location_close, *location_open;
	gboolean retval = FALSE;
	
	if (!close)
		return FALSE;
	
	if (gtk_text_buffer_get_selection_bounds (dc->buffer, &start, &end))
	{
		text = gtk_text_buffer_get_text (dc->buffer, &start, &end, FALSE);
		if (!strncmp (text, open, strlen (open)) &&
				!strncmp (text + strlen (text) - strlen (close), close, strlen (close)))
			retval = TRUE;
		g_free (text);
	}
	else
	{
		mark_insert = gtk_text_buffer_get_insert (dc->buffer);
		gtk_text_buffer_get_iter_at_mark (dc->buffer, &start, mark_insert);
		gtk_text_buffer_get_end_iter (dc->buffer, &end);
		
		text = gtk_text_buffer_get_text (dc->buffer, &start, &end, FALSE);
		
		location_close = strstr (text, close);
		location_open = strstr (text, open);
		if ((location_close && !location_open) ||
				(location_close && location_close < location_open))
			retval = TRUE;
		
		g_free (text);
	}
	
	return retval;
}

static void
handle_insert_html (DrivelClient *dc, gchar *open, gchar *close)
{
	if (inside_tag (dc, open, close))
		skip_tag (dc, close);
	else
		insert_html (dc, open, close);
	
	return;
}

static void
bold_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<b>", "</b>");
	
	return;
}

static void
italic_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<i>", "</i>");
	
	return;
}

static void
underline_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<u>", "</u>");
	
	return;
}

static void
strikethrough_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<strike>", "</strike>");
	
	return;
}

static void
superscript_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<sup>", "</sup>");
	
	return;
}

static void
subscript_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<sub>", "</sub>");
	
	return;
}

static void
list_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<ul>", "</ul>");
	
	return;
}

static void
listitem_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<li>", NULL);
	
	return;
}

static void
blockquote_cb (GtkWidget *menu, DrivelClient *dc)
{
	handle_insert_html (dc, "<blockquote>", "</blockquote>");
	
	return;
}

#if 0
static gboolean
key_press_cb (GtkWidget *text, GdkEventKey *event, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	if (event->state & GDK_CONTROL_MASK)
	{
		if (event->keyval == GDK_B || event->keyval == GDK_b)
			bold_cb (NULL, dc);
		else if (event->keyval == GDK_I || event->keyval == GDK_i)
			italic_cb (NULL, dc);
		else if (event->keyval == GDK_U || event->keyval == GDK_u)
			underline_cb (NULL, dc);
		else if (event->keyval == GDK_S || event->keyval == GDK_s)
			strikethrough_cb (NULL, dc);
	}
	
	return FALSE;
}
#endif

static void
grey_edit_menu (DrivelClient *dc)
{
	GtkClipboard *clipboard;
	
	if (gtk_text_buffer_get_selection_bounds (dc->buffer, NULL, NULL))
	{
		gtk_widget_set_sensitive (dc->menu_cut, TRUE);
		gtk_widget_set_sensitive (dc->menu_copy, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive (dc->menu_cut, FALSE);
		gtk_widget_set_sensitive (dc->menu_copy, FALSE);
	}
	
	clipboard = gtk_clipboard_get (GDK_NONE);
	if (gtk_clipboard_wait_is_text_available (clipboard))
		gtk_widget_set_sensitive (dc->menu_paste, TRUE);
	else
		gtk_widget_set_sensitive (dc->menu_paste, FALSE);
	
	return;
}

static gboolean
text_button_release_cb (GtkWidget *text, GdkEventButton *event, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	grey_edit_menu (dc);
	
	return FALSE;
}

static gboolean
text_key_release_cb (GtkWidget *text, GdkEventKey *event, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	grey_edit_menu (dc);
	
	return TRUE;
}

void
add_gconf_notifies (DrivelClient *dc)
{
	dc->id.server_id = gconf_client_notify_add (dc->client, dc->gconf->default_server, server_changed_cb,
			dc, NULL, NULL);
	dc->id.mood_id = gconf_client_notify_add (dc->client, dc->gconf->default_mood, mood_changed_cb,
			dc, NULL, NULL);
	dc->id.music_id = gconf_client_notify_add (dc->client, dc->gconf->default_music, music_changed_cb,
			dc, NULL, NULL);
	dc->id.picture_id = gconf_client_notify_add (dc->client, dc->gconf->default_picture, picture_changed_cb,
			dc, NULL, NULL);
	dc->id.security_id = gconf_client_notify_add (dc->client, dc->gconf->default_security, security_changed_cb,
			dc, NULL, NULL);
	dc->id.comment_id = gconf_client_notify_add (dc->client, dc->gconf->default_comment, comment_changed_cb,
			dc, NULL, NULL);
	dc->id.autoformat_id = gconf_client_notify_add (dc->client, dc->gconf->default_autoformat, autoformat_changed_cb,
			dc, NULL, NULL);
	dc->id.proxy_id = gconf_client_notify_add (dc->client, dc->gconf->proxy, proxy_changed_cb,
			dc, NULL, NULL);
	dc->id.proxy_user_id = gconf_client_notify_add (dc->client, dc->gconf->proxy_user, proxy_user_changed_cb,
			dc, NULL, NULL);
	dc->id.proxy_pass_id = gconf_client_notify_add (dc->client, dc->gconf->proxy_pass, proxy_pass_changed_cb,
			dc, NULL, NULL);
	dc->id.proxy_url_id = gconf_client_notify_add (dc->client, dc->gconf->proxy_url, proxy_url_changed_cb,
			dc, NULL, NULL);
	dc->id.proxy_port_id = gconf_client_notify_add (dc->client, dc->gconf->proxy_port, proxy_port_changed_cb,
			dc, NULL, NULL);
	dc->id.tray_id = gconf_client_notify_add (dc->client, dc->gconf->tray, tray_changed_cb,
			dc, NULL, NULL);
			
	return;
}

void
remove_gconf_notifies (GConfClient *client, DrivelIDs *id)
{
	gconf_client_notify_remove (client, id->server_id);
	gconf_client_notify_remove (client, id->mood_id);
	gconf_client_notify_remove (client, id->music_id);
	gconf_client_notify_remove (client, id->picture_id);
	gconf_client_notify_remove (client, id->security_id);
	gconf_client_notify_remove (client, id->comment_id);
	gconf_client_notify_remove (client, id->autoformat_id);
	gconf_client_notify_remove (client, id->proxy_id);
	gconf_client_notify_remove (client, id->proxy_user_id);
	gconf_client_notify_remove (client, id->proxy_pass_id);
	gconf_client_notify_remove (client, id->proxy_url_id);
	gconf_client_notify_remove (client, id->proxy_port_id);
	
	return;
}

static GtkWidget *
get_focused_widget (DrivelClient *dc)
{
	GtkWidget *focused;
	
	if (gtk_widget_is_focus (dc->journal_subject))
		focused = dc->journal_subject;
	else if (gtk_widget_is_focus (GTK_BIN (dc->journal_mood)->child))
		focused = GTK_BIN (dc->journal_mood)->child;
	else if (gtk_widget_is_focus (dc->journal_music))
		focused = dc->journal_music;
	else
		focused = dc->journal_text;
	
	return focused;
}

static void
cut_cb (GtkWidget *menu, gpointer data)
{
	GtkWidget *editable;
	GtkClipboard *clipboard;
	DrivelClient *dc = (DrivelClient *) data;
	
	editable = get_focused_widget (dc);
	
	if (GTK_IS_EDITABLE (editable))
		gtk_editable_cut_clipboard (GTK_EDITABLE (editable));
	else
	{
		clipboard = gtk_clipboard_get (GDK_NONE);
		gtk_text_buffer_cut_clipboard (dc->buffer, clipboard, TRUE);
	}
	
	gtk_widget_set_sensitive (dc->menu_cut, FALSE);
	gtk_widget_set_sensitive (dc->menu_copy, FALSE);
		
	return;
}

static void
copy_cb (GtkWidget *menu, gpointer data)
{
	GtkWidget *editable;
	GtkClipboard *clipboard;
	DrivelClient *dc = (DrivelClient *) data;
	
	editable = get_focused_widget (dc);
	
	if (GTK_IS_EDITABLE (editable))
		gtk_editable_copy_clipboard (GTK_EDITABLE (editable));
	else
	{
		clipboard = gtk_clipboard_get (GDK_NONE);
		gtk_text_buffer_copy_clipboard (dc->buffer, clipboard);
	}
	
	return;
}

static void
paste_cb (GtkWidget *menu, gpointer data)
{
	GtkWidget *editable;
	GtkClipboard *clipboard;
	DrivelClient *dc = (DrivelClient *) data;
	
	editable = get_focused_widget (dc);
	
	if (GTK_IS_EDITABLE (editable))
		gtk_editable_paste_clipboard (GTK_EDITABLE (editable));
	else
	{
		clipboard = gtk_clipboard_get (GDK_NONE);
		gtk_text_buffer_paste_clipboard (dc->buffer, clipboard, NULL, TRUE);
	}
	
	return;
}

static void
clear_cb (GtkWidget *menu, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	journal_display_defaults (dc);
	
	return;
}

static void
select_all_cb (GtkWidget *menu, gpointer data)
{
	GtkWidget *editable;
	GtkTextIter start, end;
	DrivelClient *dc = (DrivelClient *) data;
	
	editable = get_focused_widget (dc);
	
	if (GTK_IS_EDITABLE (editable))
		gtk_editable_select_region (GTK_EDITABLE (editable), 0, -1);
	else
	{
		gtk_text_buffer_get_bounds (dc->buffer, &start, &end);
		gtk_text_buffer_move_mark_by_name (dc->buffer, "insert", &start);
		gtk_text_buffer_move_mark_by_name (dc->buffer, "selection_bound", &end);	
	}
	
	return;
}

static void
insert_link_cb (GtkWidget *menu, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;

	display_insert_link_dialog (dc);
	
	return;
}

static void
insert_image_cb (GtkWidget *menu, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	display_insert_image_dialog (dc);
	
	return;
}

static void
insert_poll_cb (GtkWidget *menu, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	display_insert_poll_dialog (dc);	
	
	return;
}

static void
edit_entry_cb (GtkWidget *widget, gpointer data)
{
	gchar *post_data, *active_journal;
	gchar *values [3];
	gint i;
	DrivelClient *dc = (DrivelClient *) data;
	
	dc->edit_entry = TRUE;
	
	display_save_dialog (dc);
	
	values [0] = lj_format_string (dc->username);
	values [1] = lj_format_string ("-1");
	values [2] = lj_format_string (dc->active_journal);
	
	if (dc->active_journal)
		active_journal = g_strdup_printf ("&usejournal=%s", values [2]);
	else
		active_journal = g_strdup ("");
	
	post_data = g_strdup_printf (
		"mode=getevents&ver=1&user=%s&hpassword=%s&"
		"selecttype=one&itemid=%s&"
		"lineendings=unix"
		"%s",
		values [0], dc->password, values [1], active_journal);
	
	lj_send_request (dc, post_data, REQUEST_GETEVENTS);
	
	for (i = 0; i < 3; i++)
		curl_free (values [i]);

	g_free (active_journal);

	return;
}

static void
active_journal_cb (GtkRadioAction *action, GtkRadioAction *current, gpointer data)
{
	DrivelClient *dc;
	gchar *name;
	gboolean active;
	
	name = (gchar *) data;
	dc = g_object_get_data (G_OBJECT (action), "dc");
	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
	
	if (active)
	{
		g_free (dc->active_journal);
		if (!strncmp (name, dc->username, strlen (name)))
			dc->active_journal = NULL;
		else
			dc->active_journal = g_strdup (name);
	}
	
	return;
}

void
edit_preferences_cb (GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	GSList *user_list;
	DrivelClient *dc = (DrivelClient *) data;

	if (!dc->journal_window && dc->username && strcmp (dc->username, ""))
	{
		drivel_gconf_data_fill (dc->gconf, dc->username, dc->client, &dc->id, dc);

		gconf_client_set_string (dc->client, "/apps/drivel/global_settings/current_user", dc->username, NULL);

		user_list = gconf_client_get_list (dc->client, "/apps/drivel/global_settings/user_list",
				GCONF_VALUE_STRING, NULL);
		if (user_list != NULL && g_slist_find_custom (user_list, dc->username, string_compare))
		{
		}
		else
		{
			user_list = g_slist_prepend (user_list, g_strdup (dc->username));
			gconf_client_set_list (dc->client, "/apps/drivel/global_settings/user_list", GCONF_VALUE_STRING, user_list, NULL);
		}

		g_free (dc->server);
		dc->server = gconf_client_get_string (dc->client, dc->gconf->default_server, NULL);
		if (!dc->server)
			dc->server = g_strdup ("www.livejournal.com");			

		if (dc->id.server_id)
			gconf_client_notify_remove (dc->client, dc->id.server_id);
		dc->id.server_id = gconf_client_notify_add (dc->client, dc->gconf->default_server, server_changed_cb,
			dc, NULL, NULL);
	}
	else if (!dc->username || !strcmp (dc->username, ""))
	{
		dialog = gtk_message_dialog_new (
				GTK_WINDOW (dc->current_window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_MESSAGE_ERROR,
				GTK_BUTTONS_CLOSE,
				_("You must enter your username first."));
		
		gtk_dialog_run (GTK_DIALOG (dialog));
		
		gtk_widget_destroy (dialog);
		
		return;
	}
	
	display_edit_preferences_dialog (dc);
	
	return;
}

static void
edit_friends_cb (GtkWidget *widget, gpointer data)
{
	gchar *post_data;
	DrivelClient *dc = (DrivelClient *) data;
	
	post_data = g_strdup_printf (
			"mode=getfriends&ver=1&user=%s&"
			"hpassword=%s&includefriendof=1",
			dc->username, dc->password);
	
	lj_send_request (dc, post_data, REQUEST_GETFRIENDS);
	
	return;
}

static void
edit_history_cb (GtkWidget *widget, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	display_edit_history_dialog (dc);
	
	return;
}

#if 0
static void
options_cb (GtkWidget *button, gpointer data)
{
	gboolean state;
	DrivelClient *dc = (DrivelClient *) data;
	
	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
	
	if (state)
		gtk_widget_show_all (dc->options_frame);
	else
		gtk_widget_hide_all (dc->options_frame);
	
	return;
}
#endif

static void
journal_post_cb (GtkWidget *button, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	post_entry (dc, REQUEST_POSTEVENT);
	
	return;
}

static void
journal_save_cb (GtkWidget *button, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	post_entry (dc, REQUEST_EDITEVENT);
	
	return;
}

static void
expander_activate_cb (GtkWidget *expander, GParamSpec *param_spec, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;

	gconf_client_set_bool (dc->client, dc->gconf->expander_open, 
			gtk_expander_get_expanded (GTK_EXPANDER (expander)), 
			NULL);

}

static void
journal_cancel_cb (GtkWidget *button, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	
	journal_edit_entry_finished (dc);
	
	return;
}
	
static void
journal_delete_cb (GtkWidget *button, gpointer data)
{
	GtkTextIter start, end;
	GtkTextBuffer *buffer;
	DrivelClient *dc = (DrivelClient *) data;
	
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (dc->journal_text));
	
	gtk_text_buffer_get_bounds (buffer, &start, &end);
	
	gtk_text_buffer_delete (buffer, &start, &end);
	
	post_entry (dc, REQUEST_EDITEVENT);
	
	return;
}

void
journal_window_state_save (DrivelClient *dc)
{
	gboolean max;
	gint x, y, height, width;
	GdkWindowState window_state;
	
	/* save the window position and size */
	if (dc->journal_window)
	{
		gtk_window_get_position (GTK_WINDOW (dc->journal_window), &x, &y);
		gtk_window_get_size (GTK_WINDOW (dc->journal_window), &width, &height);
		window_state = gdk_window_get_state (dc->journal_window->window);
		max = (window_state & GDK_WINDOW_STATE_MAXIMIZED);
		if (!max)
		{
			gconf_client_set_int (dc->client, dc->gconf->entry_x, x, NULL);
			gconf_client_set_int (dc->client, dc->gconf->entry_y, y, NULL);
			gconf_client_set_int (dc->client, dc->gconf->entry_height, height, NULL);
			gconf_client_set_int (dc->client, dc->gconf->entry_width, width, NULL);
		}
		gconf_client_set_bool (dc->client, dc->gconf->entry_max, max, NULL);
	}
	
	return;
}

void
journal_window_state_restore (DrivelClient *dc)
{
	gboolean max;
	gint x, y, height, width;
	
	if (dc->journal_window)
	{
		x = gconf_client_get_int (dc->client, dc->gconf->entry_x, NULL);
		y = gconf_client_get_int (dc->client, dc->gconf->entry_y, NULL);
		height = gconf_client_get_int (dc->client, dc->gconf->entry_height, NULL);
		width = gconf_client_get_int (dc->client, dc->gconf->entry_width, NULL);
		max = gconf_client_get_bool (dc->client, dc->gconf->entry_max, NULL);
		
		if (height < 5)
			height = 400;
		if (width < 5)
			width = 600;
		
		gtk_window_move (GTK_WINDOW (dc->journal_window), x, y);
		gtk_window_resize (GTK_WINDOW (dc->journal_window), width, height);
		if (max)
			gtk_window_maximize (GTK_WINDOW (dc->journal_window));
		
		gtk_window_present (GTK_WINDOW (dc->journal_window));
		if (dc->current_window != dc->journal_window)
			gtk_window_present (GTK_WINDOW (dc->current_window));
	}
	
	return;
}

static gchar *
fix_mnemonic_string (const gchar *string)
{
	gchar *new_string;
	gint i, j, len, count;
	
	len = strlen (string);
	
	for (i = count = 0; i < len; i++)
		if (string [i] == '_')
			count++;
	
	new_string = g_new0 (gchar, len + count + 1);
	new_string [len + count] = '\0';
	
	for (i = j = 0; i < len; i++)
	{
		if (string [i] == '_')
			new_string [j++] = '_';
		new_string [j++] = string [i];
	}
	
	return new_string;
}

static void
build_active_journal_menu (DrivelClient *dc, GtkUIManager *ui_manager)
{
	GtkActionGroup *action_group;
	GSList *list, *group;
	gchar **ui_descriptions, *ui_description;
	gint i;
	GError *error;
	
	ui_descriptions = g_new0 (gchar *, dc->journals + 3);
	ui_descriptions[0] = g_strdup (
		"<ui>"
		"  <menubar name='MainMenu'>"
		"    <menu action='EditMenu'>"
		"      <menu action='ActiveJournal'>");
	
	action_group = gtk_action_group_new ("ActiveJournalActions");
	
	list = dc->journal_list;
	group = NULL;
	
	for (i = 0; i < dc->journals; i++)
	{
		GtkRadioAction *action;
		gchar *name, *label;
		
		name = g_strdup_printf ("ActiveJournalItem%d", i);
		label = fix_mnemonic_string (list->data);
		
		action = gtk_radio_action_new (name, label, 
				N_("Select a journal to post to"), NULL, i);
		gtk_radio_action_set_group (action, group);
		group = gtk_radio_action_get_group (action);
		g_object_set_data (G_OBJECT (action), "dc", dc);
		g_signal_connect (G_OBJECT (action), "changed",
					G_CALLBACK (active_journal_cb), list->data);
		gtk_action_group_add_action (action_group, GTK_ACTION (action));
		
		ui_descriptions[i + 1] = g_strdup_printf ("<menuitem action='%s'/>", name);
		
		list = list->next;
		
		g_free (name);
		g_free (label);
	}
	
	ui_descriptions[dc->journals + 1] = g_strdup (
		"      </menu>"
		"    </menu>"
		"  </menubar>"
		"</ui>");
	ui_descriptions[dc->journals + 2] = NULL;
	
	ui_description = g_strjoinv (NULL, ui_descriptions);
	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
	
	error = NULL;
	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error))
	{
		g_message ("building web menu failed: %s", error->message);
		g_error_free (error);
	}
	
	g_strfreev (ui_descriptions);
	
	return;
}

static void
web_menu_cb (GtkWidget *menu, gpointer data)
{
	gchar *url = (gchar *) data;
	
	gnome_url_show (url, NULL);
	
	return;
}

static gboolean
music_combo_cb (GtkWidget *music, GdkEventButton *event, gpointer data)
{
	DrivelClient *dc = (DrivelClient *) data;
	GtkListStore *music_store;
	gint active;

	active = gtk_combo_box_get_active (GTK_COMBO_BOX (dc->journal_music));
	music_store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (dc->journal_music)));
	query_music_players (music_store);
	gtk_combo_box_set_active (GTK_COMBO_BOX (dc->journal_music), active);
	if (!active)
		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (dc->journal_music)->child), "");

	return FALSE;
}

static void
music_combo_changed_cb (GtkWidget *music, gpointer user_data)
{
	if (!gtk_combo_box_get_active (GTK_COMBO_BOX (music)))
		gtk_entry_set_text (GTK_ENTRY (GTK_BIN (music)->child), "");
}

static gchar *
create_mnemonic_string (const gchar *string, gchar **mnemonics)
{
	gint i, j, len;
	gchar *new_string, *fixed_string, needle [2];
	gboolean added = FALSE;
	
	needle [1] = '\0';
	
	fixed_string = fix_mnemonic_string (string);
	len = strlen (fixed_string);
	
	new_string = g_new0 (gchar, len + 2);
	j = 0;

	if (mnemonics [0][0] == '\0')
	{
		new_string [j++] = '_';
		mnemonics [0][0] = fixed_string [0];
		added = TRUE;
	}
	
	for (i = 0; i < len; i++)
	{
		if (!added)
		{
			needle [0] = fixed_string [i];

			if (!strstr (mnemonics [0], needle))
			{
				new_string [j++] = '_';
				added = TRUE;
				mnemonics [0][strlen (mnemonics [0])] = fixed_string [i];
				mnemonics [0][strlen (mnemonics [0]) + 1] = '\0';
			}
		}
		new_string [j++] = fixed_string [i];
	}

	return new_string;
}

static void
build_web_link_menu (DrivelClient *dc, GtkUIManager *ui_manager)
{
	GSList *list;
	LJMenuItem *menu;
	GtkActionGroup *action_group;
	gint i, j, k, *submenus, n_entries, n_separators, n_submenus, n_subitems;
	gchar *mnemonics, *ui_description, **ui_descriptions, **merge_descriptions;
	GError *error;
	
	n_entries = n_separators = n_submenus = n_subitems = 0;
	
	/* count the number of non-separator menu entries */
	for (list = dc->menu_list; list; list = list->next)
	{
		menu = list->data;
		if (strcmp (menu->label, "-"))
			n_entries++;
		else
			n_separators++;
		if (menu->sub_menu)
			n_submenus++;
		if (menu->menu_index)
			n_subitems++;
	}
	
	mnemonics = g_new0 (gchar, n_entries + 1);
	mnemonics [0] = '\0';
	
	ui_descriptions = g_new0 (gchar *, n_entries + n_separators + 3);
	ui_descriptions[0] = g_strdup (
		"<ui>"
		"  <menubar name='MainMenu'>"
		"    <menu action='WebMenu'>");
	
	/* submenus holds the index of the parent menu */
	submenus = g_new0 (gint, n_submenus);
	for (i = 0; i < n_submenus; i++)
		submenus[i] = 0;
	
	merge_descriptions = g_new0 (gchar *, n_subitems);
	
	action_group = gtk_action_group_new ("WebActions");
	
	for (i = 1, j = 1, k = 0, list = dc->menu_list; list; list = list->next)
	{
		menu = list->data;
		
		if (!strcmp (menu->label, "-"))
			ui_descriptions[i] = g_strdup_printf ("<separator name=\'sep%d\'/>", i);
		else
		{
			gchar *id, *label;
			GtkAction *action;
			
			id = g_strdup_printf ("WebItem%d", j);
			label = create_mnemonic_string (menu->label, &mnemonics);
			
			action = gtk_action_new (id, label, NULL, NULL);
			gtk_action_group_add_action (action_group, action);
			
			if (!menu->sub_menu)
			{
				g_signal_connect (G_OBJECT (action), "activate",
					G_CALLBACK (web_menu_cb), menu->url);
				
				if (menu->menu_index)
				{
					gchar *parent;
					
					parent = g_strdup_printf ("WebItem%d", 
							submenus[menu->menu_index]);
					ui_descriptions[i] = g_strdup ("");
					merge_descriptions[k] = g_strdup_printf (
							"<ui><menubar name='MainMenu'>"
							"  <menu action='WebMenu'>"
							"    <menu action='%s'>"
							"      <menuitem action='%s'/>"
							"    </menu>"
							"  </menu>"
							"</menubar></ui>",
							parent, id);
					
					g_free (parent);
					
					k++;
				}
				else
				{
					ui_descriptions[i] = g_strdup_printf (
							"<menuitem action=\'%s\'/>", id);
				}
			}
			else
			{
				submenus[menu->sub_menu] = j;
				ui_descriptions[i] = g_strdup_printf (
						"<menu action=\'%s\'></menu>", id);
			}
			
			g_free (id);
			g_free (label);
			
			j++;
		}
		i++;
	}

	ui_descriptions[n_entries + n_separators + 1] = g_strdup (
		"    </menu>"
		"  </menubar>"
		"</ui>");
	ui_descriptions[n_entries + n_separators + 2] = NULL;
	
	ui_description = g_strjoinv (NULL, ui_descriptions);

	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
	
	error = NULL;
	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error))
	{
		g_message ("building web menu failed: %s", error->message);
		g_error_free (error);
	}
	for (i = 0; i < n_subitems; i++)
	{
		error = NULL;
		if (!gtk_ui_manager_add_ui_from_string (ui_manager, merge_descriptions[i], -1, &error))
		{
			g_message ("building web submenu %d failed: %s", i, error->message);
			g_error_free (error);
		}
	}
	
	g_strfreev (ui_descriptions);
	
	return;
}

void
journal_window_build (DrivelClient *dc)
{
	/*
	 * these are the menus
	 */
	static GtkActionEntry entries[] =
	{
		{ "FileMenu", NULL, N_("_Journal") },
		{ "EditMenu", NULL, N_("_Edit") },
		{ "FormatMenu", NULL, N_("_Format") },
		{ "WebMenu", NULL, N_("_Web Links"), },
		{ "HelpMenu", NULL, N_("_Help"), },
		{ "OpenDraft", GTK_STOCK_OPEN, N_("_Open Draft..."), "<control>O", N_("Open an unfinished journal entry"), G_CALLBACK (open_draft_cb) },
		{ "SaveDraft", GTK_STOCK_SAVE, N_("_Save Draft"), "<control>S", N_("Save the current text without adding it to your journal"), G_CALLBACK (save_draft_cb) },
		/* FIXME: make an icon for log-out */
		{ "LogOut", GTK_STOCK_UNDO, N_("_Log Out"), NULL, N_("Switch usernames"), G_CALLBACK (logout_cb) },
		{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (exit_cb) },
		{ "Cut", GTK_STOCK_CUT, NULL, NULL, NULL, G_CALLBACK (cut_cb) },
		{ "Copy", GTK_STOCK_COPY, NULL, NULL, NULL, G_CALLBACK (copy_cb) },
		{ "Paste", GTK_STOCK_PASTE, NULL, NULL, NULL, G_CALLBACK (paste_cb) },
		{ "Clear", GTK_STOCK_CLEAR, N_("C_lear Entry"), "<control>T", NULL, G_CALLBACK (clear_cb) },
		{ "SelectAll", NULL, N_("Select _All"), "<control>A", NULL, G_CALLBACK (select_all_cb) },
		/* FIXME: make an icon for edit-last-entry */
		{ "LastEntry", GTK_STOCK_FIND_AND_REPLACE, N_("Last _Entry"), "<control>L", N_("Edit the last entry you posted"), G_CALLBACK (edit_entry_cb) },
		{ "Friends", NULL, N_("_Friends"), NULL, N_("Edit your friends list"), G_CALLBACK (edit_friends_cb) },
		{ "History", NULL, N_("_History"), NULL, N_("View or edit a previous entry"), G_CALLBACK (edit_history_cb) },
		{ "ActiveJournal", NULL, N_("Active _Journal"), },
		{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (edit_preferences_cb) },
		{ "Bold", GTK_STOCK_BOLD, NULL, "<control>B", NULL, G_CALLBACK (bold_cb) },
		{ "Italic", GTK_STOCK_ITALIC, NULL, "<control>I", NULL, G_CALLBACK (italic_cb) },
		{ "Underline", GTK_STOCK_UNDERLINE, NULL, "<control>U", NULL, G_CALLBACK (underline_cb) },
		{ "Strike", GTK_STOCK_STRIKETHROUGH, NULL, NULL, NULL, G_CALLBACK (strikethrough_cb) },
		{ "Super", NULL, N_("Supe_rscript"), NULL, NULL, G_CALLBACK (superscript_cb) },
		{ "Sub", NULL, N_("_Subscript"), NULL, NULL, G_CALLBACK (subscript_cb) },
		{ "List", NULL, N_("L_ist"), NULL, NULL, G_CALLBACK (list_cb) },
		{ "ListItem", NULL, N_("List Ite_m"), NULL, NULL, G_CALLBACK (listitem_cb) },
		{ "Indent", NULL, N_("I_ndent"), NULL, NULL, G_CALLBACK (blockquote_cb) },
		{ "InsertLink", "drivel-insert-link", N_("Insert _Link..."), NULL, N_("Create a link to a web page, livejournal user, or a long journal entry"), G_CALLBACK (insert_link_cb) },
		{ "InsertImage", "drivel-insert-image", N_("Insert Ima_ge..."), NULL, N_("Insert an image into your journal entry"), G_CALLBACK (insert_image_cb) },
		{ "InsertPoll", NULL, N_("Insert _Poll..."), NULL, NULL, G_CALLBACK (insert_poll_cb) },
		{ "About", GNOME_STOCK_ABOUT, N_("_About"), NULL, NULL, G_CALLBACK (about_cb) }
	};
	static const gchar *ui_description =
	{
		"<ui>"
		"  <menubar name='MainMenu'>"
		"    <menu action='FileMenu'>"
		"      <menuitem action='OpenDraft'/>"
		"      <menuitem action='SaveDraft'/>"
		"      <separator name='sep1'/>"
		"      <menuitem action='LogOut'/>"
		"      <menuitem action='Quit'/>"
		"    </menu>"
		"    <menu action='EditMenu'>"
		"      <menuitem action='Cut'/>"
		"      <menuitem action='Copy'/>"
		"      <menuitem action='Paste'/>"
		"      <menuitem action='Clear'/>"
		"      <separator name='sep2'/>"
		"      <menuitem action='SelectAll'/>"
		"      <separator name='sep3'/>"
		"      <menuitem action='LastEntry'/>"
		"      <menuitem action='Friends'/>"
		"      <menuitem action='History'/>"
		"      <menu action='ActiveJournal'>"
		"      </menu>"
		"      <separator name='sep4'/>"
		"      <menuitem action='Preferences'/>"
		"    </menu>"
		"    <menu action='FormatMenu'>"
		"      <menuitem action='Bold'/>"
		"      <menuitem action='Italic'/>"
		"      <menuitem action='Underline'/>"
		"      <menuitem action='Strike'/>"
		"      <menuitem action='Super'/>"
		"      <menuitem action='Sub'/>"
		"      <separator name='sep5'/>"
		"      <menuitem action='List'/>"
		"      <menuitem action='ListItem'/>"
		"      <separator name='sep6'/>"
		"      <menuitem action='Indent'/>"
		"      <separator name='sep7'/>"
		"      <menuitem action='InsertLink'/>"
		"      <menuitem action='InsertImage'/>"
		"      <menuitem action='InsertPoll'/>"
		"    </menu>"
		"    <menu action='WebMenu'>"
		"    </menu>"
		"    <menu action='HelpMenu'>"
		"      <menuitem action='About'/>"
		"    </menu>"
		"  </menubar>"
		"</ui>"
	};

	/*
	 * this is the new glade interface for the journal
	 * written by Davyd Madeley <davyd@ucc.asn.au>
	 */
	GladeXML *xml;
	gchar *title, *glade_path;
	GtkWidget *window, *subject, *security, *mood, *music, *comment, *autoformat, *picture;
	GtkWidget *text_area, *menubar, *expander, *mood_icon, *music_icon;
	GtkWidget *post_button, *cancel_button, *delete_button, *save_button, *help_button;
	GtkListStore *security_store, *picture_store, *mood_store, *music_store;
	GtkTreeIter iter;
	GtkCellRenderer *renderer;
	GdkPixbuf *pixbuf;
	GtkTextBuffer *buffer;
	GSList *list;
	GtkUIManager *ui_manager;
	GtkActionGroup *action_group;
	GtkAccelGroup *accel_group;
	GError *error;
	
	/* try the current directory first, for debugging */
	xml = glade_xml_new (DRIVEL_GLADE_FILE, NULL, GETTEXT_PACKAGE);
	if (!xml)
	{
		glade_path = g_build_filename (DRIVEL_GLADE_DIR, DRIVEL_GLADE_FILE, NULL);
		xml = glade_xml_new (glade_path, NULL, GETTEXT_PACKAGE);
		g_free (glade_path);
		
		if (!xml)
			return;
	}

	/* grab and retitle the window */
	title = g_strdup_printf (_("Drivel - %s's Journal"), dc->name);
	window = glade_xml_get_widget (xml, "journal_editor");
	gtk_window_set_title (GTK_WINDOW (window), title);
	g_free (title);
	gtk_window_set_icon_from_file (GTK_WINDOW (window),
			DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S
			"livejournal.png", NULL);
	
	/* subject entry */
	subject = glade_xml_get_widget (xml, "subject");
	
	/* the security dropdown, populate with entries and pictures */
	security_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	gtk_list_store_append (security_store, &iter);
	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "public.png", NULL);
	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Public"), -1);
	gtk_list_store_append (security_store, &iter);
	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "protected.png", NULL);
	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Friends-Only"), -1);
	gtk_list_store_append (security_store, &iter);
	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps"
			G_DIR_SEPARATOR_S "drivel" G_DIR_SEPARATOR_S "private.png", NULL);
	gtk_list_store_set (security_store, &iter, 0, pixbuf, 1, _("Private"), -1);
	
	security = glade_xml_get_widget (xml, "security");
	gtk_combo_box_set_model (GTK_COMBO_BOX (security), GTK_TREE_MODEL (security_store));
	gtk_cell_layout_clear (GTK_CELL_LAYOUT (security));
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (security), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (security), renderer,
			"pixbuf", 0,
			NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (security), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (security), renderer,
			"text", 1,
			NULL);
	
	/* the text area */
	text_area = glade_xml_get_widget (xml, "textview");
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_area));

	/* the mood entry */
	mood_icon = glade_xml_get_widget (xml, "mood_icon");
	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "drivel"
			G_DIR_SEPARATOR_S "mood.png", NULL);
	pixbuf = gdk_pixbuf_scale_simple (pixbuf, 24, 24, GDK_INTERP_BILINEAR);
	gtk_image_set_from_pixbuf (GTK_IMAGE (mood_icon), pixbuf);
	
	mood_store = gtk_list_store_new (1, G_TYPE_STRING);
	dc->mood_list = g_slist_sort (dc->mood_list, string_compare);
	for (list = dc->mood_list; list != NULL; list = list->next)
	{
		gtk_list_store_append (mood_store, &iter);
		gtk_list_store_set (mood_store, &iter, 0, list->data, -1);
	}
	mood = glade_xml_get_widget (xml, "mood");
	gtk_combo_box_set_model (GTK_COMBO_BOX (mood), GTK_TREE_MODEL (mood_store));
	gtk_cell_layout_clear (GTK_CELL_LAYOUT (mood));
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (mood), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (mood), renderer,
			"text", 0,
			NULL);

	/* the music entry */
	music_icon = glade_xml_get_widget (xml, "music_icon");
	pixbuf = gdk_pixbuf_new_from_file (DATADIR G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "drivel"
			G_DIR_SEPARATOR_S "music.png", NULL);
	pixbuf = gdk_pixbuf_scale_simple (pixbuf, 16, 16, GDK_INTERP_BILINEAR);
	gtk_image_set_from_pixbuf (GTK_IMAGE (music_icon), pixbuf);
	
	music_store = gtk_list_store_new (2, G_TYPE_STRING, GDK_TYPE_PIXBUF);
	music = glade_xml_get_widget (xml, "music");
	gtk_combo_box_set_model (GTK_COMBO_BOX (music), GTK_TREE_MODEL (music_store));
	gtk_cell_layout_clear (GTK_CELL_LAYOUT (music));
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (music), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (music), renderer,
			"pixbuf", 1,
			NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (music), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (music), renderer,
			"text", 0,
			NULL);

	/* no comments check */
	comment = glade_xml_get_widget (xml, "nocomments");
	/* no autoformatting check */
	autoformat = glade_xml_get_widget (xml, "noautoformat");

	/* pictures dropdown */
	picture_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	fill_picture_menu (dc, picture_store);
	picture = glade_xml_get_widget (xml, "picture");
	gtk_combo_box_set_model (GTK_COMBO_BOX (picture), GTK_TREE_MODEL (picture_store));
	gtk_cell_layout_clear (GTK_CELL_LAYOUT (picture));
	renderer = gtk_cell_renderer_pixbuf_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (picture), renderer, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (picture), renderer,
			"pixbuf", 0,
			NULL);
	renderer = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (picture), renderer, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (picture), renderer,
			"text", 1,
			NULL);
	dc->picture_store = picture_store;
	
	/* buttons */
	help_button = glade_xml_get_widget (xml, "help_button");
	delete_button = glade_xml_get_widget (xml, "delete_button");
	cancel_button = glade_xml_get_widget (xml, "cancel_button");
	save_button = glade_xml_get_widget (xml, "save_button");
	post_button = glade_xml_get_widget (xml, "post_button");
	gtk_button_set_use_stock (GTK_BUTTON (post_button), TRUE);

	/* store things in the struct */
	dc->journal_text = text_area;
	dc->journal_window = window;
	dc->journal_subject = subject;
	dc->journal_mood = mood;
	dc->journal_music = music;
	dc->journal_security = security;
	dc->journal_picture = picture;
	dc->journal_comment = comment;
	dc->journal_autoformat = autoformat;
	dc->journal_post = post_button;
	dc->modified = FALSE;
	dc->buffer = buffer;
	dc->edit_delete = delete_button;
	dc->edit_cancel = cancel_button;
	dc->edit_save = save_button;

	drivel_push_current_window (dc, window);

	/* menubar */
	action_group = gtk_action_group_new ("MenuActions");
	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), dc);
	ui_manager = gtk_ui_manager_new ();
	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
	accel_group = gtk_ui_manager_get_accel_group (ui_manager);
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

	error = NULL;
	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error))
	{
		g_message ("building menus failed: %s", error->message);
		g_error_free (error);
	}
	build_active_journal_menu (dc, ui_manager);
	build_web_link_menu (dc, ui_manager);
	menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
	gnome_app_set_menus (GNOME_APP (window), GTK_MENU_BAR (menubar));
	
	dc->menu_cut = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/EditMenu/Cut");
	dc->menu_copy = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/EditMenu/Copy");
	dc->menu_paste = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/EditMenu/Paste");

	/* set the expander state */
	expander = glade_xml_get_widget(xml, "more_options");
	gtk_expander_set_expanded (GTK_EXPANDER (expander), gconf_client_get_bool (dc->client, dc->gconf->expander_open, NULL));

	/* signals */
	g_signal_connect (G_OBJECT (window), "delete_event",
			G_CALLBACK (delete_event_cb), dc);
	g_signal_connect (G_OBJECT (buffer), "changed",
			G_CALLBACK (text_changed_cb), dc);
	g_signal_connect (G_OBJECT (text_area), "button_release_event",
			G_CALLBACK (text_button_release_cb), dc);
	g_signal_connect (G_OBJECT (post_button), "clicked",
			G_CALLBACK (journal_post_cb), dc);
	g_signal_connect (G_OBJECT (delete_button), "clicked",
			G_CALLBACK (journal_delete_cb), dc);
	g_signal_connect (G_OBJECT (cancel_button), "clicked",
			G_CALLBACK (journal_cancel_cb), dc);
	g_signal_connect (G_OBJECT (save_button), "clicked",
			G_CALLBACK (journal_save_cb), dc);
	g_signal_connect (G_OBJECT (expander), "notify::expanded",
			G_CALLBACK (expander_activate_cb), dc);
	/* FIXME: this is not the right callback */
	g_signal_connect (G_OBJECT (music), "motion_notify_event",
			G_CALLBACK (music_combo_cb), dc);
	g_signal_connect (G_OBJECT (music), "changed",
			G_CALLBACK (music_combo_changed_cb), dc);

	/* FIXME: these workarounds are related to GNOME bug #141183 */
	gtk_label_set_mnemonic_widget (GTK_LABEL (glade_xml_get_widget (xml, "mood_label")), mood);
	gtk_label_set_mnemonic_widget (GTK_LABEL (glade_xml_get_widget (xml, "music_label")), music);
	gtk_label_set_mnemonic_widget (GTK_LABEL (glade_xml_get_widget (xml, "picture_label")), picture);
	
	dc->tag_autosave = g_timeout_add (5000, autosave_cb, dc);

	text_key_release_cb (NULL, NULL, dc);

	journal_display_defaults (dc);
	/* load the autosave file (if available) */
	open_file (NULL, dc);
	
	if (gconf_client_get_bool (dc->client, dc->gconf->tray, NULL))
		tray_create (dc);

	journal_window_state_restore (dc);

	gtk_widget_show_all (window);

	/* this is hidden for the time being, as we don't have any help */
	gtk_widget_hide (help_button);
	gtk_widget_hide (delete_button);
	gtk_widget_hide (cancel_button);
	gtk_widget_hide (save_button);
	
	gtk_window_present (GTK_WINDOW (window));
	if (dc->message_dialog)
	{
		gtk_window_set_transient_for (GTK_WINDOW (dc->message_dialog), 
				GTK_WINDOW (window));
		gtk_widget_hide (dc->message_dialog);
		gtk_window_present (GTK_WINDOW (dc->message_dialog));
	}
	
	if (gconf_client_get_bool (dc->client, dc->gconf->min_start, NULL))
		gtk_window_iconify (GTK_WINDOW (dc->journal_window));
	
	return;
}
